#include "include.h"

void crossProduct(float *vResult, float *VecA, float *VecB);
void rotateVector(float rad, float *vec, float x, float y, float z);
void calcPlane(float *vResult, float *point1, float *point2, float *point3);
void drawPlane(float *plane, float *point, GLfloat fExtent, GLfloat fStep);
float dotProduct(float *VecA, float *VecB);
void normalize(float *vec);
float magnitude(float *vec);
void drawText(const char *string, GLfloat x, GLfloat y);


void updateCamera(frame_t *camera, POINT *mousePos)
{
	static POINT	oldPos = {0};
	POINT		mouseDelta;
	float		vRight[3];

#ifdef WIN32
	// temp key handler for windows
#define SPEED 2.0f
	crossProduct(vRight, camera->up, camera->forward);
	if (GetAsyncKeyState(VK_UP))
	{
		camera->pos[0] -= camera->forward[0] * SPEED;
		camera->pos[1] -= camera->forward[1] * SPEED;
		camera->pos[2] -= camera->forward[2] * SPEED;
	}
	if (GetAsyncKeyState(VK_DOWN))
	{
		camera->pos[0] += camera->forward[0] * SPEED;
		camera->pos[1] += camera->forward[1] * SPEED;
		camera->pos[2] += camera->forward[2] * SPEED;
	}
	if (GetAsyncKeyState(VK_LEFT))
	{
		camera->pos[0] -= vRight[0] * SPEED;
		camera->pos[2] -= vRight[2] * SPEED;
	}
	if (GetAsyncKeyState(VK_RIGHT))
	{
		camera->pos[0] += vRight[0] * SPEED;
		camera->pos[2] += vRight[2] * SPEED;
	}
	if (GetAsyncKeyState(VK_RETURN))
	{
		camera->pos[1] += SPEED;
	}
	if (GetAsyncKeyState(VK_SHIFT))
	{
		camera->pos[1] -= SPEED;
	}
#endif

	if (mousePos->x == oldPos.x && mousePos->y == oldPos.y)
		return;

	mouseDelta.x = mousePos->x - oldPos.x;
	mouseDelta.y = mousePos->y - oldPos.y;
	oldPos = *mousePos;


//"camera"
	// Left / Right
	rotateVector(-mouseDelta.x / 50.0f, camera->forward, 0.0f, 1.0f, 0.0f);
	rotateVector(-mouseDelta.x / 50.0f, camera->up, 0.0f, 1.0f, 0.0f);
	normalize(camera->forward);
	normalize(camera->up);

	// Up / Down
	crossProduct(vRight, camera->up, camera->forward);
	normalize(vRight);

	rotateVector(-mouseDelta.y / 50.0f, camera->forward, vRight[0], vRight[1], vRight[2]);
	rotateVector(-mouseDelta.y / 50.0f, camera->up, vRight[0], vRight[1], vRight[2]);
	normalize(camera->forward);
	normalize(camera->up);
}

void updateCamera2(frame_t *camera, POINT *mousePos)
{
	float		vRight[3];
	frame_t		temp = {0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 0.0f, 1.0f};

	// Left / Right
	rotateVector(-mousePos->x / 50.0f, temp.forward, 0.0f, 1.0f, 0.0f);
	normalize(temp.forward);

	// Up / Down
	crossProduct(vRight, temp.up, temp.forward);
	normalize(vRight);
	rotateVector(-mousePos->y / 50.0f, temp.forward, vRight[0], vRight[1], vRight[2]);
	rotateVector(-mousePos->y / 50.0f, temp.up, vRight[0], vRight[1], vRight[2]);
	normalize(temp.forward);
	normalize(temp.up);

	camera->forward[0] = temp.forward[0];
	camera->forward[1] = temp.forward[1];
	camera->forward[2] = temp.forward[2];

	camera->up[0] = temp.up[0];
	camera->up[1] = temp.up[1];
	camera->up[2] = temp.up[2];
}

void setPos(frame_t *camera)
{
	float	matrix[16];
	float	vRight[3];

	crossProduct(vRight, camera->up, camera->forward);
	normalize(vRight);

	matrix[0] = vRight[0];
	matrix[1] = camera->up[0];
	matrix[2] = camera->forward[0];
	matrix[3] = 0.0f;

	matrix[4] = vRight[1];
	matrix[5] = camera->up[1];
	matrix[6] = camera->forward[1];
	matrix[7] = 0.0f;

	matrix[8] = vRight[2];
	matrix[9] = camera->up[2];
	matrix[10] = camera->forward[2];
	matrix[11] = 0.0f;

	matrix[12] = -(vRight[2] * camera->pos[2] + vRight[1] * camera->pos[1] + vRight[0] * camera->pos[0]);
	matrix[13] = -(camera->up[2] * camera->pos[2] + camera->up[1] * camera->pos[1] + camera->up[0] * camera->pos[0]);
	matrix[14] = -(camera->forward[2] * camera->pos[2] + camera->forward[1] * camera->pos[1] + camera->forward[0] * camera->pos[0]);
	matrix[15] = 1.0f;
	glLoadMatrixf(matrix);
}

void genFrustum(frame_t *camera, frustum_t *frustum)
{
	float	hNear, wNear, hFar, wFar;
	float	aspectRatio, zNear, zFar, fov;
	float	farCoord[4][3], nearCoord[4][3], farCenter[3], nearCenter[3];
	float	vRight[3];


	crossProduct(vRight, camera->up, camera->forward);

	aspectRatio = 1.6f;
	zNear = -1.0f;
	zFar = -2001.0f;
	fov = 45.0f;

	hNear = 2.0f * tan(fov / 2.0f) * zNear;
	wNear = hNear * aspectRatio;

	hFar = 2.0f * tan(fov / 2.0f) * zFar;
	wFar = hFar * aspectRatio;

	// far center
	farCenter[0] = camera->pos[0] + camera->forward[0] * zFar;
	farCenter[1] = camera->pos[1] + camera->forward[1] * zFar;
	farCenter[2] = camera->pos[2] + camera->forward[2] * zFar;

	// near center
	nearCenter[0] = camera->pos[0] + camera->forward[0] * zNear;
	nearCenter[1] = camera->pos[1] + camera->forward[1] * zNear;
	nearCenter[2] = camera->pos[2] + camera->forward[2] * zNear;

	// far coords
	//upper right
	farCoord[0][0] = farCenter[0] + camera->up[0] * hFar / 2.0f + vRight[0] * wFar / 2.0f;
	farCoord[0][1] = farCenter[1] + camera->up[1] * hFar / 2.0f + vRight[1] * wFar / 2.0f;
	farCoord[0][2] = farCenter[2] + camera->up[2] * hFar / 2.0f + vRight[2] * wFar / 2.0f;
	//upper left
	farCoord[1][0] = farCenter[0] + camera->up[0] * hFar / 2.0f - vRight[0] * wFar / 2.0f;
	farCoord[1][1] = farCenter[1] + camera->up[1] * hFar / 2.0f - vRight[1] * wFar / 2.0f;
	farCoord[1][2] = farCenter[2] + camera->up[2] * hFar / 2.0f - vRight[2] * wFar / 2.0f;
	//lower right
	farCoord[2][0] = farCenter[0] - camera->up[0] * hFar / 2.0f + vRight[0] * wFar / 2.0f;
	farCoord[2][1] = farCenter[1] - camera->up[1] * hFar / 2.0f + vRight[1] * wFar / 2.0f;
	farCoord[2][2] = farCenter[2] - camera->up[2] * hFar / 2.0f + vRight[2] * wFar / 2.0f;
	//lower left
	farCoord[3][0] = farCenter[0] - camera->up[0] * hFar / 2.0f - vRight[0] * wFar / 2.0f;
	farCoord[3][1] = farCenter[1] - camera->up[1] * hFar / 2.0f - vRight[1] * wFar / 2.0f;
	farCoord[3][2] = farCenter[2] - camera->up[2] * hFar / 2.0f - vRight[2] * wFar / 2.0f;

	// near coords
	//upper right
	nearCoord[0][0] = nearCenter[0] + camera->up[0] * hNear / 2.0f + vRight[0] * wNear / 2.0f;
	nearCoord[0][1] = nearCenter[1] + camera->up[1] * hNear / 2.0f + vRight[1] * wNear / 2.0f;
	nearCoord[0][2] = nearCenter[2] + camera->up[2] * hNear / 2.0f + vRight[2] * wNear / 2.0f;
	//upper left
	nearCoord[1][0] = nearCenter[0] + camera->up[0] * hNear / 2.0f - vRight[0] * wNear / 2.0f;
	nearCoord[1][1] = nearCenter[1] + camera->up[1] * hNear / 2.0f - vRight[1] * wNear / 2.0f;
	nearCoord[1][2] = nearCenter[2] + camera->up[2] * hNear / 2.0f - vRight[2] * wNear / 2.0f;
	//lower right
	nearCoord[2][0] = nearCenter[0] - camera->up[0] * hNear / 2.0f + vRight[0] * wNear / 2.0f;
	nearCoord[2][1] = nearCenter[1] - camera->up[1] * hNear / 2.0f + vRight[1] * wNear / 2.0f;
	nearCoord[2][2] = nearCenter[2] - camera->up[2] * hNear / 2.0f + vRight[2] * wNear / 2.0f;
	//lower left
	nearCoord[3][0] = nearCenter[0] - camera->up[0] * hNear / 2.0f - vRight[0] * wNear / 2.0f;
	nearCoord[3][1] = nearCenter[1] - camera->up[1] * hNear / 2.0f - vRight[1] * wNear / 2.0f;
	nearCoord[3][2] = nearCenter[2] - camera->up[2] * hNear / 2.0f - vRight[2] * wNear / 2.0f;


	glBegin(GL_LINES);
	glColor3f(1.0f, 0.0f, 0.0f);
	glVertex3fv(nearCoord[0]);
	glVertex3fv(nearCoord[1]);
	glVertex3fv(nearCoord[2]);
	glVertex3fv(nearCoord[3]);
	glVertex3fv(nearCoord[0]);
	glVertex3fv(nearCoord[2]);
	glVertex3fv(nearCoord[1]);
	glVertex3fv(nearCoord[3]);

	glColor3f(0.0f, 0.0f, 1.0f);
	glVertex3fv(farCoord[0]);
	glVertex3fv(farCoord[1]);
	glVertex3fv(farCoord[2]);
	glVertex3fv(farCoord[3]);
	glVertex3fv(farCoord[0]);
	glVertex3fv(farCoord[2]);
	glVertex3fv(farCoord[1]);
	glVertex3fv(farCoord[3]);

	glColor3f(1.0f, 1.0f, 1.0f);
	glVertex3fv(nearCoord[0]);
	glVertex3fv(farCoord[0]);
	glVertex3fv(nearCoord[1]);
	glVertex3fv(farCoord[1]);
	glVertex3fv(nearCoord[2]);
	glVertex3fv(farCoord[2]);
	glVertex3fv(nearCoord[3]);
	glVertex3fv(farCoord[3]);
	glEnd();

	// be sure normals point into frustum
	calcPlane(frustum->left, farCoord[0], nearCoord[0], nearCoord[2]);
	calcPlane(frustum->right, farCoord[1], nearCoord[3], nearCoord[1]);
	calcPlane(frustum->bottom, farCoord[0], nearCoord[1], nearCoord[0]);
	calcPlane(frustum->top, farCoord[2], nearCoord[2], nearCoord[3]);
	calcPlane(frustum->zNear, nearCoord[0], nearCoord[1], nearCoord[2]);
	calcPlane(frustum->zFar, farCoord[0], farCoord[2], farCoord[1]);
}

int inFrustum(frustum_t *frustum, float x, float y, float z)
{
	int left, right, top, bottom, zNear, zFar;

	left	= frustum->left[0] * x + frustum->left[1] * y + frustum->left[2] * z - frustum->left[3];
	right	= frustum->right[0] * x + frustum->right[1] * y + frustum->right[2] * z - frustum->right[3];
	top	= frustum->top[0] * x + frustum->top[1] * y + frustum->top[2] * z - frustum->top[3];
	bottom	= frustum->bottom[0] * x + frustum->bottom[1] * y + frustum->bottom[2] * z - frustum->bottom[3];
	zNear	= frustum->zNear[0] * x + frustum->zNear[1] * y + frustum->zNear[2] * z - frustum->zNear[3];
	zFar	= frustum->zFar[0] * x + frustum->zFar[1] * y + frustum->zFar[2] * z - frustum->zFar[3];

	{
		char buffer[80];
		sprintf(buffer, "L%d R%d T%d B%d N%d F%d", left, right, top, bottom, zNear, zFar);
		glColor3f(1.0f, 1.0f, 1.0f);
		drawText(buffer, 0.1f, 0.1f);
	}

	if ( left > 0 && right > 0 && top > 0 && bottom > 0  && zNear > 0 && zFar > 0)
		return 1;
	else
		return 0;
}

void drawPlane(float *plane, float *point, GLfloat fExtent, GLfloat fStep)
{
	float iLine, angle;
	float vRot[3], vUp[3] = {0.0f, 1.0f, 0.0f};

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();

	angle = acos(plane[1] / magnitude(plane));
	crossProduct(vRot, vUp, plane);
	glRotatef(angle * 180.0f / M_PI, vRot[0], vRot[1], vRot[2]);
	glTranslatef(point[0], point[1], point[2]);

	glBegin(GL_LINES);
	for(iLine = -fExtent; iLine <= fExtent; iLine += fStep)
	{
		glVertex3f(iLine, plane[3], fExtent);
		glVertex3f(iLine, plane[3], -fExtent);
		glVertex3f(fExtent, plane[3], iLine);
		glVertex3f(-fExtent, plane[3], iLine);
	}
	glEnd();
	glPopMatrix();
}

void genFrustumT(frustum_t *frustum, float *clip)
{

	frustum->right[0] = clip[ 3] - clip[ 0];
	frustum->right[1] = clip[ 7] - clip[ 4];
	frustum->right[2] = clip[11] - clip[ 8];
	frustum->right[3] = clip[15] - clip[12];

	frustum->left[0] = clip[ 3] + clip[ 0];
	frustum->left[1] = clip[ 7] + clip[ 4];
	frustum->left[2] = clip[11] + clip[ 8];
	frustum->left[3] = clip[15] + clip[12];

	frustum->bottom[0] = clip[ 3] + clip[ 1];
	frustum->bottom[1] = clip[ 7] + clip[ 5];
	frustum->bottom[2] = clip[11] + clip[ 9];
	frustum->bottom[3] = clip[15] + clip[13];

	frustum->top[0] = clip[ 3] - clip[ 1];
	frustum->top[1] = clip[ 7] - clip[ 5];
	frustum->top[2] = clip[11] - clip[ 9];
	frustum->top[3] = clip[15] - clip[13];

	frustum->zFar[0] = clip[ 3] - clip[ 2];
	frustum->zFar[1] = clip[ 7] - clip[ 6];
	frustum->zFar[2] = clip[11] - clip[10];
	frustum->zFar[3] = clip[15] - clip[14];

	frustum->zNear[0] = clip[ 3] + clip[ 2];
	frustum->zNear[1] = clip[ 7] + clip[ 6];
	frustum->zNear[2] = clip[11] + clip[10];
	frustum->zNear[3] = clip[15] + clip[14];
}

void genFrustumM(frustum_t *frustum, float *matrix)
{
/*
0 [m11 m12 m13 m14]
4 [m21 m22 m23 m24]
8 [m31 m32 m33 m34]
12[m41 m42 m43 m44]
*/

	frustum->left[0] = matrix[12] + matrix[0];
	frustum->left[1] = matrix[13] + matrix[1];
	frustum->left[2] = matrix[14] + matrix[2];
	frustum->left[3] = matrix[15] + matrix[3];

	frustum->right[0] = matrix[12] - matrix[0];
	frustum->right[1] = matrix[13] - matrix[1];
	frustum->right[2] = matrix[14] - matrix[2];
	frustum->right[3] = matrix[15] - matrix[3];

	frustum->bottom[0] = matrix[12] + matrix[4];
	frustum->bottom[1] = matrix[13] + matrix[5];
	frustum->bottom[2] = matrix[14] + matrix[6];
	frustum->bottom[3] = matrix[15] + matrix[7];

	frustum->top[0] = matrix[12] - matrix[4];
	frustum->top[1] = matrix[13] - matrix[5];
	frustum->top[2] = matrix[14] - matrix[6];
	frustum->top[3] = matrix[15] - matrix[7];

	frustum->zNear[0] = matrix[12] + matrix[8];
	frustum->zNear[1] = matrix[13] + matrix[9];
	frustum->zNear[2] = matrix[14] + matrix[10];
	frustum->zNear[3] = matrix[15] + matrix[11];

	frustum->zFar[0] = matrix[12] + matrix[8]; 
	frustum->zFar[1] = matrix[13] + matrix[9]; 
	frustum->zFar[2] = matrix[14] + matrix[10];
	frustum->zFar[3] = matrix[15] + matrix[11];
}
