#include "include.h"
void renderMain(frame_t *, bsp_t *, texObjects_t *);
void resize(GLint, GLint);
void updateCamera(frame_t *, POINT *);
void updateCamera2(frame_t *, POINT *);
void loadTextures(bsp_t *map, texObjects_t *texObjects);
void loadEntities(bsp_t *map);
void playsound(wave_t *);
int loadWav(char *, wave_t *);
void initMap(bsp_t *map);
void setupHighLevelShaders(void);

GLuint myProgram;

// The idea here is to use this to hold static info and be platform independant in EngineProc handlers
void EngineProc(unsigned int message, int *param1, int *param2)
{
#ifdef _WIN32
	static HWND	hwnd;
	static HDC	hdc;
#else
	static Display	*display;
	static Window	window;	
	XFontStruct	*font;
#endif

	static POINT		mousePos;

	static bsp_t		*map;
	static texObjects_t	texObjects;
	static frame_t		camera = { 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f, 0.0f, 1.0f};
	static ALCdevice	*soundDevice;
	static ALCcontext	*soundContext;
	static wave_t		wave;

	static int		init = 0;

	if (message != E_INIT && !init)
		return;

	switch (message)
	{
	case E_INIT:
		init = 1;
	// Windowing system
		#ifdef _WIN32
			hwnd = (HWND)*param1;
			hdc = (HDC)*param2;
		#else
			display = (Display *)param1;
			window = *((Window *)param2);
		#endif

	// Graphics
		#ifdef _WIN32
			wglUseFontBitmaps(hdc, ' ', '~', 1000);
		#else
			font = XLoadQueryFont(display, "-*-courier-bold-r-normal--14-*-*-*-*-*-*-*");
			if (font)
				glXUseXFont(font->fid, ' ', '~', 1000);
			else
				printf("Unable to load font!\n");
		#endif

		glClearColor(0.25f, 0.25f, 0.25f, 1.0f);
		glEnable(GL_DEPTH_TEST);
		glPointSize(5.0);
		glEnable(GL_CULL_FACE);
		glFrontFace(GL_CW);
		glEnableClientState(GL_VERTEX_ARRAY);
		glEnableClientState(GL_TEXTURE_COORD_ARRAY);
		glEnableClientState(GL_NORMAL_ARRAY);
		glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
		glEnable(GL_COLOR_MATERIAL);
		glShadeModel(GL_SMOOTH);
//		setupHighLevelShaders();
		map = (bsp_t *)getFile("Media/Maps/map.bsp");
		initMap(map);
		loadEntities(map);
		loadTextures(map, &texObjects);

	// Sound
		//deviceList = alcGetString(NULL, ALC_DEVICE_SPECIFIER);
		soundDevice = alcOpenDevice(NULL);
		soundContext = alcCreateContext(soundDevice, NULL);
		alcMakeContextCurrent(soundContext);
		alGetError();
		loadWav("Media/Sound/MWmusic.wav", &wave);
//		playsound(&wave);
		break;

	case E_RENDER:
		if (init == 0)
			break;
		updateCamera(&camera, &mousePos);
		renderMain(&camera, map, &texObjects);
		#ifdef _WIN32
			SwapBuffers(hdc);
		#else
			glXSwapBuffers(display, window);
		#endif
		break;

	case E_MOUSEPOS:
		mousePos.x = *param1;
		mousePos.y = *param2;
		break;

	case E_RESIZE:
		resize( *param1, *param2 );
		break;

	case E_DEINIT:
		#ifdef _WIN32
			// ;o)
		#else
			if (font)
				XUnloadFont(display, font->fid);
		#endif
		glDeleteLists(1000, '~' - ' ');
		alcMakeContextCurrent(NULL);
		alcDestroyContext(soundContext);
		alcCloseDevice(soundDevice);
		glDeleteTextures(2 * texObjects.nObjects, texObjects.texObject);
		free((void *)texObjects.texObject);
		free((void *)wave.file);
		free((void *)map);
		break;
	}
}
