#include "include.h"

void EngineProc(unsigned int, int *, int *);
void setPos(frame_t *);
void drawAxis(frame_t *camera);
char *getFile(char *fileName);
int inFrustum(frustum_t *frustum, float x, float y, float z);
void renderMap(bsp_t *map, texObjects_t *texObjects, frame_t *camera, frustum_t *frustum);
void drawText(const char *string, GLfloat x, GLfloat y);
void debugFrustum(frame_t *camera, frustum_t *frustum);
void debugMatrix(void);
void genFrustum(frame_t *camera, frustum_t *frustum);
void crossProduct(float *vResult, float *VecA, float *VecB);
void normalize(float *vec);


extern GLuint myProgram;

void renderMain(frame_t *camera, bsp_t *map, texObjects_t *texObjects)
{
	frustum_t	frustum;

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	setPos(camera);
	drawAxis(camera);
	genFrustum(camera, &frustum);
//	glUseProgram(myProgram);
	renderMap(map, texObjects, camera, &frustum);
//	glUseProgram(0);
//	debugMatrix();
}

void drawAxis(frame_t *camera)
{
	float		vRight[3];
	frustum_t	little;

	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
		glLoadIdentity();
		glOrtho(0.0f, 1.33f, 0.0f, 1.0f, -1.0f, 1.0f);
		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();
			glLoadIdentity();
			glTranslatef(1.2f, 0.1f, 0.0f);
			glScalef(0.05f, 0.05f, 0.05f);
			crossProduct(vRight, camera->up, camera->forward);
			glColor3f(1.0f, 0.0f, 0.0f);
			glBegin(GL_LINES);
				glVertex3f(0.0f, 0.0f, 0.0f);
				glVertex3f(camera->forward[0], camera->forward[1], camera->forward[2]);
			glEnd();
		
			glColor3f(0.0f, 1.0f, 0.0f);
			glBegin(GL_LINES);
				glVertex3f(0.0f, 0.0f, 0.0f);
				glVertex3f(camera->up[0], camera->up[1], camera->up[2]);
			glEnd();
		
			glColor3f(0.0f, 0.0f, 1.0f);
			glBegin(GL_LINES);
				glVertex3f(0.0f, 0.0f, 0.0f);
				glVertex3f(vRight[0], vRight[1], vRight[2]);
			glEnd();
			glScalef(0.001f, 0.001f, 0.001f);
			genFrustum(camera, &little);
		glPopMatrix();
		glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
}

void resize(GLint width, GLint height)
{
	glViewport(0, 0, width, height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	if (!height)
		height = 1;
	gluPerspective( 45.0, (double) width / height, 1.0, 2001.0 );
	glMatrixMode(GL_MODELVIEW);
}

void setupHighLevelShaders(void)
{
	GLuint		myVertexShader, myPixelShader;
	const GLchar	*myStringPtrs[1];
	GLchar		*sVertexShader, *sPixelShader;
	FILE		*fLog;
	GLint		success;

	fLog = fopen("infolog.txt", "w");
	
	sVertexShader = getFile("Media/Shaders/glsl/mlighting.vs");
	if (sVertexShader == NULL)
	{
		fprintf(fLog, "Unable to load vertexShader");
		fclose(fLog);
		return;
	} else {
		fprintf(fLog, "Loaded vertex shader\n");
	}

	myStringPtrs[0] = sVertexShader;
	myVertexShader = glCreateShader(GL_VERTEX_SHADER);
	glShaderSource(myVertexShader, 1, myStringPtrs, NULL);
	glCompileShader(myVertexShader);
	glGetShaderiv(myVertexShader, GL_COMPILE_STATUS, &success);
	if (!success)
	{
		GLint		infoSize;
		GLchar		*infoLog;

		glGetShaderiv(myVertexShader, GL_INFO_LOG_LENGTH, &infoSize);
		infoLog = (GLchar *)malloc(sizeof(GLchar) * infoSize);

		glGetShaderInfoLog(myVertexShader, infoSize, NULL, infoLog);
		fprintf(fLog, "Error compiling: %s", infoLog);
		fclose(fLog);
		return;
	}


	sPixelShader = getFile("Media/Shaders/glsl/mlighting.fs");
	if (sPixelShader == NULL)
	{
		fprintf(fLog, "Unable to load pixelShader");
		fclose(fLog);
		return;
	} else {
		fprintf(fLog, "Loaded fragment shader\n");
	}
	myStringPtrs[0] = sPixelShader;
	myPixelShader = glCreateShader(GL_FRAGMENT_SHADER);
	glShaderSource(myPixelShader, 1, myStringPtrs, NULL);
	glCompileShader(myPixelShader);
	glGetShaderiv(myPixelShader, GL_COMPILE_STATUS, &success);
	if (!success)
	{
		GLint		infoSize;
		GLchar		*infoLog;

		glGetShaderiv(myPixelShader, GL_INFO_LOG_LENGTH, &infoSize);
		infoLog = (GLchar *)malloc(sizeof(GLchar) * infoSize);

		glGetShaderInfoLog(myPixelShader, infoSize, NULL, infoLog);
		fprintf(fLog, "Error compiling: %s", infoLog);
		fclose(fLog);
		free((void *)infoLog);
		return;
	}

	myProgram = glCreateProgram();
	glAttachShader(myProgram, myVertexShader);
	glAttachShader(myProgram, myPixelShader);
	glLinkProgram(myProgram);
	glGetProgramiv(myProgram, GL_LINK_STATUS, &success);
	if (!success)
	{
		GLint		infoSize;
		GLchar		*infoLog;

		glGetProgramiv(myProgram, GL_INFO_LOG_LENGTH, &infoSize);
		infoLog = (GLchar *)malloc(sizeof(GLchar) * infoSize);

		glGetProgramInfoLog(myProgram, infoSize, NULL, infoLog);
		fprintf(fLog, "Error linking: %s", infoLog);
		fclose(fLog);
		free((void *)infoLog);
		return;
	} else {
		fprintf(fLog, "Link successful\n");
	}
	fclose(fLog);
}

void drawBox(int *min, int *max)
{
	glDisable(GL_LIGHTING);
	glDisable(GL_TEXTURE_2D);
	glBegin(GL_LINES);
	//bottom square
	glVertex3i(min[0], min[2], -min[1]);
	glVertex3i(max[0], min[2], -min[1]);
	glVertex3i(min[0], min[2], -min[1]);
	glVertex3i(min[0], max[2], -min[1]);
	glVertex3i(max[0], max[2], -min[1]);
	glVertex3i(max[0], min[2], -min[1]);
	glVertex3i(max[0], max[2], -min[1]);
	glVertex3i(min[0], max[2], -min[1]);
	//top square                     
	glVertex3i(min[0], min[2], -max[1]);
	glVertex3i(max[0], min[2], -max[1]);
	glVertex3i(min[0], min[2], -max[1]);
	glVertex3i(min[0], max[2], -max[1]);
	glVertex3i(max[0], max[2], -max[1]);
	glVertex3i(max[0], min[2], -max[1]);
	glVertex3i(max[0], max[2], -max[1]);
	glVertex3i(min[0], max[2], -max[1]);
	//remaining legs                 
	glVertex3i(min[0], min[2], -min[1]);
	glVertex3i(min[0], min[2], -max[1]);
	glVertex3i(min[0], max[2], -max[1]);
	glVertex3i(min[0], max[2], -min[1]);
                                         
	glVertex3i(max[0], min[2], -min[1]);
	glVertex3i(max[0], min[2], -max[1]);
	glVertex3i(max[0], max[2], -max[1]);
	glVertex3i(max[0], max[2], -min[1]);
	glEnd();
	glEnable(GL_LIGHTING);
	glEnable(GL_TEXTURE_2D);
}

void drawText(const char *string, GLfloat x, GLfloat y)
{
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
		glLoadIdentity();
		glOrtho(0.0f, 1.0f, 0.0f, 1.0f, 0.0f, 0.1f);
		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();
			glLoadIdentity();
			glRasterPos2f(x, y);
			glListBase(1000 - ' ');
			glCallLists(strlen(string), GL_UNSIGNED_BYTE, string);	
		glPopMatrix();
		glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
}

void debugMatrix(void)
{
	float matrix[16];
	char buffer[1024];

	glGetFloatv(GL_MODELVIEW_MATRIX , matrix);
	glColor3f(1.0f, 1.0f, 1.0f);
	sprintf(buffer, "[%3.3f %3.3f %3.3f %3.3f]", matrix[0], matrix[1], matrix[2], matrix[3]);
	drawText(buffer, 0.1f, 0.9f);
	sprintf(buffer, "[%3.3f %3.3f %3.3f %3.3f]", matrix[4], matrix[5], matrix[6], matrix[7]);
	drawText(buffer, 0.1f, 0.85f);
	sprintf(buffer, "[%3.3f %3.3f %3.3f %3.3f]", matrix[8], matrix[9], matrix[10], matrix[11]);
	drawText(buffer, 0.1f, 0.8f);
	sprintf(buffer, "[%3.3f %3.3f %3.3f %3.3f]", matrix[12], matrix[13], matrix[14], matrix[15]);
	drawText(buffer, 0.1f, 0.75f);
}

void debugFrustum(frame_t *camera, frustum_t *frustum)
{
	int left, right, top, bottom, zNear, zFar, inside;
	float x, y, z;
	x = y = z = 0.0f;

	left	= frustum->left[0] * x + frustum->left[1] * y + frustum->left[2] * z - frustum->left[3];
	right	= frustum->right[0] * x + frustum->right[1] * y + frustum->right[2] * z - frustum->right[3];
	top	= frustum->top[0] * x + frustum->top[1] * y + frustum->top[2] * z - frustum->top[3];
	bottom	= frustum->bottom[0] * x + frustum->bottom[1] * y + frustum->bottom[2] * z - frustum->bottom[3];
	zNear	= frustum->zNear[0] * x + frustum->zNear[1] * y + frustum->zNear[2] * z - frustum->zNear[3];
	zFar	= frustum->zFar[0] * x + frustum->zFar[1] * y + frustum->zFar[2] * z - frustum->zFar[3];

	if ( left > 0 && right > 0 && top > 0 && bottom > 0  && zNear > 0 && zFar > 0)
		inside = 1;
	else
		inside = 0;

	{
		char buffer[80];

		glColor3f(1.0f, 1.0f, 1.0f);
		sprintf(buffer, "[%d] L%d R%d T%d B%d N%d F%d", inside, left, right, top, bottom, zNear, zFar);
		drawText(buffer, 0.1f, 0.1f);
		sprintf(buffer, "position: %f %f %f", camera->pos[0], camera->pos[1], camera->pos[2]);
		drawText(buffer, 0.1f, 0.2f);
		sprintf(buffer, "forward: %f %f %f", camera->forward[0], camera->forward[1], camera->forward[2]);
		drawText(buffer, 0.1f, 0.15f);

	}
}
