#ifndef INCLUDE_H
#define INCLUDE_H

#ifdef _WIN32
	#define _USE_MATH_DEFINES
	#define WIN32_EXTRA_LEAN
	#define _CRT_SECURE_NO_DEPRECATE
	#define NOMINMAX
	#include <windows.h>

	//gfx
	#include <GL/glew.h>

	//physx
//	#include <NxPhysics.h>	// C++ required for ageia sdk, trying to stay C'ish if possible
//	#include <NxCooking.h>
#else
	#include <GL/gl.h>
	#include <GL/glu.h>
	#include <GL/glx.h>
	#include <X11/Xlib.h>
	#include <X11/Xatom.h>
	#include <sys/select.h>
	#include <sys/types.h>
	#include <sys/time.h>
#endif

//audio
#include <AL/al.h>
#include <AL/alc.h>

//std
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "map.h"	// bsp structures

//engineProc
#define E_INIT		1
#define E_RENDER	2
#define	E_MOUSEPOS	3
#define	E_RESIZE	4
#define E_DEINIT	5

//PhysicsProc
#define P_INIT		1
#define P_RENDER	2
#define P_TIMESTEP	3
#define P_LOADBRUSHES	4
#define P_DEINIT	5


// I dont want to have to prototype this everywhere
GLbyte *gltLoadTGA(const char *szFileName, GLint *iWidth, GLint *iHeight, GLint *iComponents, GLenum *eFormat);
char *getFile(char *fileName);

typedef struct
{
	short	format;
	short	channels;
	int	sampleRate;
	int	avgSampleRate;
	short	align;
	short	sampleSize;
} waveFormat_t;

typedef struct
{
	waveFormat_t	*format;
	void		*pcmData;
	int		dataSize;
	int		duration;
	char		*file;
} wave_t;

typedef struct
{
	float	pos[3], up[3], forward[3];
} frame_t;

typedef struct
{
	float	left[4];
	float	right[4];
	float	top[4];
	float	bottom[4];
	float	zNear[4];
	float	zFar[4];
} frustum_t;

typedef struct
{
	int		*texObject;
	int		nObjects;
	// add stuff as needed
} texObjects_t;

#ifndef _WIN32
typedef struct
{
	int x, y;
} POINT;
#endif


#endif
