#include "include.h"

void processEnt(char *entity);
void addLight(float *pos, float *color, int intensity);
void drawBox(int *min, int *max);
int isClusterVisible(int visCluster, int testCluster, tBSPVisData *visData);
int findLeaf(bsp_t *map, frame_t *frame);
void normalize(float *vec);

void renderMap(bsp_t *map, texObjects_t *texObjects, frame_t *camera, frustum_t *frustum)
{
	tBSPLeaf	*frameLeaf, *baseLeaf, *leaf;
	tBSPVisData	*visData;
	tBSPFace	*baseFace, *face;
	tBSPVertex	*vert;
	char		*sMap = (char *)map;
	int		i, j, cameraOffset, nLeafs, nFaces, faceIndex;
	int		*leafFace, *meshVert;

	// init all needed bsp data
	leafFace = (int *)&sMap[map->directory[kLeafFaces].offset];
	baseLeaf = (tBSPLeaf *)&sMap[map->directory[kLeafs].offset];
	baseFace = (tBSPFace *)&sMap[map->directory[kFaces].offset];
	visData = (tBSPVisData *)&sMap[map->directory[kVisData].offset];
	vert = (tBSPVertex *)(sMap + map->directory[kVertices].offset);
	meshVert = (int *)&sMap[map->directory[kMeshVerts].offset];

	// Find and index the camera leaf
	cameraOffset = findLeaf(map, camera);
	frameLeaf = &baseLeaf[cameraOffset];

	//cycle through all leafs comparing vis data to camera leaf cluster
	nLeafs = map->directory[kLeafs].length / sizeof(tBSPLeaf);
	for (i = nLeafs - 1; i >= 0; i--)
	{
		leaf = &baseLeaf[i];


		if ( !isClusterVisible(frameLeaf->cluster, leaf->cluster, visData) )
			continue;

/*
		if ( !inFrustum(frustum, (float)leaf->mins[0], (float)leaf->mins[1], (float)leaf->mins[2]) )
			if ( !inFrustum(frustum, (float)leaf->maxs[0], (float)leaf->maxs[1], (float)leaf->maxs[2]) )
				continue;
*/


		nFaces = leaf->numOfLeafFaces;
		for (j = nFaces - 1; j >= 0; j--)
		{
			faceIndex = leafFace[leaf->leafface + j];
			face = &baseFace[faceIndex];

			// TODO: check if face has already been rendered and act accordingly

			if (face->type == 2)
			{
				// bezier patch
			}
			else if (face->type == 4)
			{
				// billboard
			}
			else
			{
				// init gl display pointers
				glVertexPointer(3, GL_FLOAT, sizeof(tBSPVertex), &(vert[face->vertexIndex].vPosition) );
				glTexCoordPointer(2, GL_FLOAT, sizeof(tBSPVertex), &(vert[face->vertexIndex].vTextureCoord) );
				glNormalPointer(GL_FLOAT, sizeof(tBSPVertex), &(vert[face->vertexIndex].vNormal) );

//				glActiveTexture(GL_TEXTURE0);
				glEnable(GL_TEXTURE_2D);
				glBindTexture(GL_TEXTURE_2D, texObjects->texObject[face->textureID]);
//				glActiveTexture(GL_TEXTURE1);
//				glEnable(GL_TEXTURE_2D);
//				glBindTexture(GL_TEXTURE_2D, texObjects->texObject[face->textureID + texObjects->nObjects / 2]);
				if (face->type == 1)
					glColor3f(1.0f, 1.0f, 1.0f);
				else
					glColor3f(0.0f, 0.0f, 0.0f);
				glEnable(GL_LIGHTING);
				glDrawElements(GL_TRIANGLES, face->numMeshVerts, GL_UNSIGNED_INT, &meshVert[face->meshVertIndex]);
				glDisable(GL_LIGHTING);
				glDisable(GL_TEXTURE_2D);
//				glActiveTexture(GL_TEXTURE0);
//				glDisable(GL_TEXTURE_2D);
			}
		}
		if (cameraOffset == i)
		{
			glColor3f(0.0f, 1.0f, 0.0f);
			drawBox(leaf->mins, leaf->maxs);
		} else {
			glColor3f(1.0f, 0.0f, 0.0f);
			drawBox(leaf->mins, leaf->maxs);
		}
	}
}

void renderPatch()
{
	
}

void loadTextures(bsp_t *map, texObjects_t *texObjects)
{
	tBSPTexture	*bspTex;
	char		*pBytes, *pMap;
	int		iWidth, iHeight, iComponents;
	GLenum		eFormat;
	int		offset, length;
	int		i;

	pMap = (char *)map;

	offset = map->directory[kTextures].offset;
	length = map->directory[kTextures].length;
	texObjects->nObjects = length / sizeof(tBSPTexture);
	texObjects->texObject = (int *)malloc(2 * texObjects->nObjects * sizeof(int));

	glGenTextures(2 * texObjects->nObjects, texObjects->texObject);
	for (i = 0; i < texObjects->nObjects; i++ )
	{
		char buffer[80];

		bspTex = (tBSPTexture *) ( pMap + offset + i * sizeof(tBSPTexture) );
		strcpy(buffer, bspTex->strName);
		strcat(buffer, ".tga");
		glBindTexture(GL_TEXTURE_2D, texObjects->texObject[i]);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
		glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_TRUE);
		pBytes = gltLoadTGA(buffer, &iWidth, &iHeight, &iComponents, &eFormat);
		glTexImage2D(GL_TEXTURE_2D, 0, iComponents, iWidth, iHeight, 0, eFormat, GL_UNSIGNED_BYTE, pBytes);
		free(pBytes);

// temp - load normals
		strcpy(buffer, bspTex->strName);
		strcat(buffer, "_normal.tga");
		glBindTexture(GL_TEXTURE_2D, texObjects->texObject[i + texObjects->nObjects]);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
		glTexParameteri(GL_TEXTURE_2D, GL_GENERATE_MIPMAP, GL_TRUE);
		pBytes = gltLoadTGA(buffer, &iWidth, &iHeight, &iComponents, &eFormat);
		glTexImage2D(GL_TEXTURE_2D, 0, iComponents, iWidth, iHeight, 0, eFormat, GL_UNSIGNED_BYTE, pBytes);
		free(pBytes);
	}
	texObjects->nObjects *= 2;
}

void loadEntities(bsp_t *map)
{
	char *pMap, *sEntities, *start, *end;
	int i;

	pMap = (char *)map;
	sEntities = pMap + map->directory[kEntities].offset;
	if (sEntities[0] != '{')
		return;

	for(i = 1; i < map->directory[kEntities].length;)
	{
		char	*entity;
		int length;

		start = strstr(sEntities + i, "{");
		if (!start)
			break;
		end = strstr(start, "}");
		length = ++end - start;
		entity = (char *)malloc(length + 1);
		strncpy(entity, start, length);
		entity[length] = '\0';
		processEnt(entity);
		free(entity);
		i += end - start;
	}
}

void processEnt(char *entity)
{
	if ( strstr(entity, "\"classname\" \"light\"") )
	{
		char	*sOrigin, *sIntensity, *sColor;
		int	bright;
		float	pos[4], color[4] = {1.0f};

		sOrigin = strstr(entity, "\"origin\" \"");
		if (sOrigin)
		{
			int x, y, z;

			sscanf(sOrigin + 10, "%d %d %d", &x, &y, &z);
			pos[0] = (float)x;
			pos[1] = (float)z;
			pos[2] = (float)-y;
		}
		else
			return;

		sIntensity = strstr(entity, "\"light\" \"");
		if (sIntensity)
			sscanf(sIntensity + 9, "%d", &bright);
		else
			return;

		sColor = strstr(entity, "\"_color\" \"");
		if (sColor)
			sscanf(sColor + 10, "%f %f %f", &color[0], &color[1], &color[2]);

		pos[3] = 1.0f;
		addLight(pos, color, bright);
	}
}

void addLight(float *pos, float *color, int intensity)
{
	static int	nLights = 0;
	GLfloat		lightSpecular[]	= {0.8f, 0.8f, 1.0f, 0.5f};
	GLenum	light[8] = { GL_LIGHT0, GL_LIGHT1, GL_LIGHT2, GL_LIGHT3, GL_LIGHT4, GL_LIGHT5, GL_LIGHT6, GL_LIGHT7 };

	if (nLights > 8)
		return;

	glLightfv(light[nLights], GL_AMBIENT, color);
	glLightfv(light[nLights], GL_DIFFUSE, color);
	glLightfv(light[nLights], GL_POSITION, pos);
	glLightfv(GL_LIGHT0, GL_SPECULAR, lightSpecular);

	glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL, GL_SEPARATE_SPECULAR_COLOR);
	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
	glMaterialfv(GL_FRONT, GL_SPECULAR, lightSpecular);
	glMateriali(GL_FRONT, GL_SHININESS, 16);


	glEnable(light[nLights]);
	nLights++;
}

//  ints (visCluster, testCluster) are indexes into vis data (bit index or byte index?)
int isClusterVisible(int visCluster, int testCluster, tBSPVisData *visData)
{
	int byteOffset, bitOffset;
	char testByte;

	// if no vis data or if camera leaf invalid (outside map) draw regardless
	if (visCluster < 0)
		return 1;
	// bit of pVecs we need to return
	bitOffset = visCluster + testCluster * visData->vectorSize * 8; 
	byteOffset = bitOffset / 8;
	testByte = 1 << (bitOffset - byteOffset * 8);
	return 	(&visData->pVecs)[byteOffset] & testByte;
}

int findLeaf(bsp_t *map, frame_t *frame)
{
	tBSPNode	*baseNode, *node;
	tBSPPlane	*basePlane, *plane;
	char		*sMap;
	float		distance;
	int		i = 0;

	sMap = (char *)map;
	baseNode = (tBSPNode *)(sMap + map->directory[kNodes].offset);
	basePlane = (tBSPPlane *)(sMap + map->directory[kPlanes].offset);

	while(i >= 0)
	{
		node = &baseNode[i];
		plane = &basePlane[node->plane];
		distance = plane->vNormal[0] * frame->pos[0] + plane->vNormal[1] * frame->pos[1] + plane->vNormal[2] * frame->pos[2] - plane->d;

		if(distance >= 0)	
			i = baseNode[i].front;
		else
			i = baseNode[i].back;
	}
	return -(i + 1);
}

void initMap(bsp_t *map)
{
	char		*sMap = (char *)map;
	int		i, nVerts, nPlanes;
	tBSPVertex	*baseVert, *vert;
	tBSPPlane	*basePlane, *plane;
	float		temp;


/*
	map->baseVert = (tBSPVertex *)&sMap[map->directory[kVertices].offset];
	map->baseEnt = (char *)&sMap[map->directory[kEntities].offset];
	map->baseTexture = (tBSPTexture *)&sMap[map->directory[kTextures].offset];
	map->basePlane = (tBSPPlane *)&sMap[map->directory[kPlanes].offset];
	map->baseNode = (tBSPNode *)&sMap[map->directory[kNodes].offset];
	map->baseLeaf = (tBSPLeaf *)&sMap[map->directory[kLeafs].offset];
	map->baseLeafFace = (int *)&sMap[map->directory[kLeafFaces].offset];
	map->baseLeafBrush = (int *)&sMap[map->directory[kLeafBrushes].offset];
	map->baseVert = (tBSPVertex *)&sMap[map->directory[kVertices].offset];
	map->baseFace = (tBSPFace *)&sMap[map->directory[kFaces].offset];
	map->baseVis = (tBSPVisData*)&sMap[map->directory[kVisData].offset];
*/
	baseVert = (tBSPVertex *)&sMap[map->directory[kVertices].offset];
	nVerts = map->directory[kVertices].length / sizeof(tBSPVertex);
	for(i = 0; i < nVerts; i++)
	{
		vert = &baseVert[i];

		temp = vert->vPosition[1];
		vert->vPosition[1] = vert->vPosition[2];
		vert->vPosition[2] =  -temp;
	}

	basePlane = (tBSPPlane *)&sMap[map->directory[kPlanes].offset];
	nPlanes = map->directory[kPlanes].length / sizeof(tBSPVertex);
	for(i = 0; i < nPlanes; i++)
	{
		plane = &basePlane[i];

		temp = plane->vNormal[1];
		plane->vNormal[1] = plane->vNormal[2];
		plane->vNormal[2] =  -temp;
	}
}
