/*
	references:	http://graphics.stanford.edu/~kekoa/q3/ -q3 bsp file format
			http://graphics.cs.brown.edu/games/quake/quake3.html -rendering bsp's 'sequel' to above
*/

#ifndef _WIN32
	typedef char byte;
#endif
typedef unsigned char ubyte;


typedef struct
{
	char	strID[4];	 // This should always be 'IBSP'
	int	version;	 // This should be 0x2e for Quake 3 files
} tBSPHeader;

typedef struct
{
	int	offset;		// Offset to start of lump, relative to beginning of file. 
	int	length;		// Length of lump. Always a multiple of 4. 
} tBSPLump;

typedef struct
{
	tBSPHeader	header;
	tBSPLump	directory[17];
/*
	char		*baseEnt;
	tBSPTexture	*baseTexture;
	tBSPPlane	*basePlane;
	tBSPNode	*baseNode;
	tBSPLeaf	*baseLeaf;
	int		*baseLeafFace;
	int		*baseLeafBrush;
	tBSPVertex	*baseVert;
	tBSPFace	*baseFace;
	tBSPVisData	*baseVis;
*/
} bsp_t;

typedef enum
{
	kEntities = 0,		// Stores player/object positions, etc...
	kTextures,		// Stores texture information
	kPlanes,		// Stores the splitting planes
	kNodes,			// Stores the BSP nodes
	kLeafs,			// Stores the leafs of the nodes
	kLeafFaces,		// Stores the leaf's indices into the faces
	kLeafBrushes,		// Stores the leaf's indices into the brushes
	kModels,		// Stores the info of world models
	kBrushes,		// Stores the brushes info (for collision)
	kBrushSides,		// Stores the brush surfaces info
	kVertices,		// Stores the level vertices
	kMeshVerts,		// Stores the model vertices offsets - this is just wrong, these are the indicies for the face verticies. Try rendering q3map2 bsps.
	kShaders,		// Stores the shader files (blending, anims..)
	kFaces,			// Stores the faces for the level
	kLightmaps,		// Stores the lightmaps for the level
	kLightVolumes,		// Stores extra world lighting information
	kVisData,		// Stores PVS and cluster info (visibility)
	kMaxLumps		// A constant to store the number of lumps
} eLumps;

typedef struct
{
	float	vPosition[3];		// (x, y, z) position. 
	float	vTextureCoord[2];	// (u, v) texture coordinate
	float	vLightmapCoord[2];	// (u, v) lightmap coordinate
	float	vNormal[3];		// (x, y, z) normal vector
	byte	color[4];		// RGBA color for the vertex 
} tBSPVertex;

typedef struct
{
	int	textureID;		// The index into the texture array 
	int	effect;			// The index for the effects (or -1 = n/a) 
	int	type;			// 1=polygon, 2=patch, 3=mesh, 4=billboard 
	int	vertexIndex;		// The index into this face's first vertex 
	int	numOfVerts;		// The number of vertices for this face 
	int	meshVertIndex;		// The index into the first meshvertex 
	int	numMeshVerts;		// The number of mesh vertices 
	int	lightmapID;		// The texture index for the lightmap 
	int	lMapCorner[2];		// The face's lightmap corner in the image 
	int	lMapSize[2];		// The size of the lightmap section 
	float	lMapPos[3];		// The 3D origin of lightmap. 
	float	lMapVecs[2][3]; 	// The 3D space for s and t unit vectors. 
	float	vNormal[3];		// The face normal. 
	int	size[2];		// The bezier patch dimensions. 
} tBSPFace;

typedef struct
{
	char	strName[64];	// The name of the texture w/o the extension 
	int	flags;		// The surface flags (unknown) 
	int	contents;	// The content flags (unknown)
} tBSPTexture;

typedef struct
{
	byte	imageBits[128][128][3];	// The RGB data in a 128x128 image
} tBSPLightmap;

typedef struct
{
	int	plane;		// The index into the planes array 
	int	front;		// The child index for the front node 
	int	back;		// The child index for the back node 
	int	mins[3];	// The bounding box min position. 
	int	maxs[3];	// The bounding box max position. 
} tBSPNode;

typedef struct
{
	int	cluster;		// The visibility cluster 
	int	area;			// The area portal 
	int	mins[3];		// The bounding box min position 
	int	maxs[3];		// The bounding box max position 
	int	leafface;		// The first index into the face array 
	int	numOfLeafFaces;		// The number of faces for this leaf 
	int	leafBrush;		// The first index for into the brushes 
	int	numOfLeafBrushes;	// The number of brushes for this leaf 
} tBSPLeaf;

typedef struct
{
	float	vNormal[3];	// Plane normal. 
	float	d;		// The plane distance from origin 
} tBSPPlane;

typedef struct
{
	int	numOfVectors;	// This stores the number of bit-vectors
	int	vectorSize;	// The size of bit-vectors in bytes
	byte	pVecs;		// This holds all of the cluster bits
} tBSPVisData;

typedef struct 
{
	int	brushSide;		// The starting brush side for the brush 
	int	numOfBrushSides;	// Number of brush sides for the brush
	int	textureID;		// The texture index for the brush
} tBSPBrush;

typedef struct  
{
	int	plane;			// The plane index
	int	textureID;		// The texture index
} tBSPBrushSide;

typedef struct  
{
	float	min[3];		// The min position for the bounding box
	float	max[3];		// The max position for the bounding box. 
	int	faceIndex;	// The first face index in the model 
	int	numOfFaces;	// The number of faces in the model 
	int	brushIndex;	// The first brush index in the model 
	int	numOfBrushes;	// The number brushes for the model
} tBSPModel;

typedef struct
{
	char	strName[64];	// The name of the shader file 
	int	brushIndex;	// The brush index for this shader 
	int	unknown;	// This is 99% of the time 5
} tBSPShader;

typedef struct
{
	ubyte	ambient[3];	// This is the ambient color in RGB
	ubyte	directional[3];	// This is the directional color in RGB
	ubyte	direction[2];	// The direction of the light: [phi,theta] 
} tBSPLights;
