#include "include.h"

float tsin(float rad)
{
	return pow(rad, 5) / 120.f + -pow(rad, 3) / 6.0f + rad;
}

float tcos(float rad)
{
	return pow(rad, 4) / 24.0f + -rad * rad / 2.0f + 1.0f;
}

void rotateVector(float rad, float *vec, float x, float y, float z)
{
	float old[3], matrix[8], sinVal, cosVal, minusVal;

	sinVal = sin(rad);
	cosVal = cos(rad);
	minusVal = 1.0f - cosVal;

	old[0] = vec[0];
	old[1] = vec[1];
	old[2] = vec[2];

	matrix[0] = cosVal + minusVal * x * x;
	matrix[1] = minusVal * x * y - sinVal * z;
	matrix[2] = minusVal * x * z + sinVal * y;

	matrix[3] = minusVal * y * x + sinVal * z;
	matrix[4] = cosVal + minusVal * y * y;
	matrix[5] = minusVal * y * z - sinVal * x;

	matrix[6] = minusVal * z * x - sinVal * y;
	matrix[7] = minusVal * z * y + sinVal * x;
	matrix[8] = cosVal + minusVal * z * z;

	vec[0] = old[0] * matrix[0] + old[1] * matrix[1] + old[2] * matrix[2];
	vec[1] = old[0] * matrix[3] + old[1] * matrix[4] + old[2] * matrix[5]; 
	vec[2] = old[0] * matrix[6] + old[1] * matrix[7] + old[2] * matrix[8];
}

float magnitude(float *vec)
{
	return sqrt(vec[0] * vec[0] + vec[1] * vec[1] + vec[2] * vec[2]);
}

void normalize(float *vec)
{
	float length;

	length = magnitude(vec);
	if (length)
	{
		vec[0] /= length;
		vec[1] /= length;
		vec[2] /= length;
	}
	else
	{
		vec[0] = 0.0f;
		vec[1] = 0.0f;
		vec[2] = 0.0f;
	}
}

void crossProduct(float *vResult, float *VecA, float *VecB)
{
	vResult[0] = VecA[1] * VecB[2] - VecA[2] * VecB[1];
	vResult[1] = VecA[2] * VecB[0] - VecA[0] * VecB[2];
	vResult[2] = VecA[0] * VecB[1] - VecA[1] * VecB[0];
}

void calcPlane(float *vResult, float *point1, float *point2, float *point3)
{
	float	vecA[3], vecB[3];

	vecA[0] = point2[0] - point1[0];
	vecA[1] = point2[1] - point1[1];
	vecA[2] = point2[2] - point1[2];

	vecB[0] = point3[0] - point1[0];
	vecB[1] = point3[1] - point1[1];
	vecB[2] = point3[2] - point1[2];
	crossProduct(vResult, vecA, vecB);
	normalize(vResult);
	vResult[3] = -(vResult[0] * point1[0] + vResult[1] * point1[1] + vResult[2] * point1[2]);
}

void rotateVectorEuler(float *vec, float alpha, float beta, float gamma)
{
	float old[3], matrix[9], sinAlpha, sinBeta, sinGamma, cosAlpha, cosBeta, cosGamma;

	sinAlpha = sin(alpha);
	cosAlpha = cos(alpha);
	sinBeta = sin(beta);
	cosBeta = cos(beta);
	sinGamma = sin(gamma);
	cosGamma = cos(gamma);

	matrix[0] = cosAlpha * cosGamma - sinAlpha * cosBeta * sinGamma;
	matrix[1] = -sinAlpha * cosGamma - cosAlpha * cosBeta * sinGamma;
	matrix[2] = sinBeta * sinGamma;

	matrix[3] = cosAlpha * sinGamma + sinAlpha * cosBeta * cosGamma;
	matrix[4] = -sinAlpha * sinGamma + cosAlpha * cosBeta * cosGamma;
	matrix[5] = -sinBeta * cosGamma;

	matrix[6] = sinAlpha * sinBeta;
	matrix[7] = cosAlpha * sinBeta;
	matrix[8] = cosBeta;

	old[0] = vec[0];
	old[1] = vec[1];
	old[2] = vec[2];

	vec[0] = old[0] * matrix[0] + old[1] * matrix[1] + old[2] * matrix[2];
	vec[1] = old[0] * matrix[3] + old[1] * matrix[4] + old[2] * matrix[5]; 
	vec[2] = old[0] * matrix[6] + old[1] * matrix[7] + old[2] * matrix[8];
}

void rotateVectorX(float rad, float *vec) //roll
{
	float	old[3],
		matrix[9] = {	1.0f, 0.0f, 0.0f,
				0.0f, 0.0f, 0.0f,
				0.0f, 0.0f, 0.0f	};

	matrix[4] = cos(rad);
	matrix[8] = matrix[4];
	matrix[7] = sin(rad);
	matrix[5] = -matrix[7];

	old[0] = vec[0];
	old[1] = vec[1];
	old[2] = vec[2];

	vec[0] = old[0] * matrix[0] + old[1] * matrix[1] + old[2] * matrix[2];
	vec[1] = old[0] * matrix[3] + old[1] * matrix[4] + old[2] * matrix[5]; 
	vec[2] = old[0] * matrix[6] + old[1] * matrix[7] + old[2] * matrix[8];
}

void rotateVectorY(float rad, float *vec) //pitch
{
	float	old[3],
		matrix[9] = {	0.0f, 0.0f, 0.0f,
				0.0f, 1.0f, 0.0f,
				0.0f, 0.0f, 0.0f	};

	matrix[0] = cos(rad);
	matrix[8] = matrix[0];
	matrix[2] = sin(rad);
	matrix[6] = -matrix[2];

	old[0] = vec[0];
	old[1] = vec[1];
	old[2] = vec[2];

	vec[0] = old[0] * matrix[0] + old[1] * matrix[1] + old[2] * matrix[2];
	vec[1] = old[0] * matrix[3] + old[1] * matrix[4] + old[2] * matrix[5]; 
	vec[2] = old[0] * matrix[6] + old[1] * matrix[7] + old[2] * matrix[8];
}

void rotateVectorZ(float rad, float *vec) //yaw
{
	float	old[3],
		matrix[9] = {	0.0f, 0.0f, 0.0f,
				0.0f, 0.0f, 0.0f,
				0.0f, 0.0f, 1.0f	};

	matrix[0] = cos(rad);
	matrix[4] = matrix[0];
	matrix[3] = sin(rad);
	matrix[1] = -matrix[3];

	old[0] = vec[0];
	old[1] = vec[1];
	old[2] = vec[2];

	vec[0] = old[0] * matrix[0] + old[1] * matrix[1] + old[2] * matrix[2];
	vec[1] = old[0] * matrix[3] + old[1] * matrix[4] + old[2] * matrix[5]; 
	vec[2] = old[0] * matrix[6] + old[1] * matrix[7] + old[2] * matrix[8];
}

/*
//"world"
	// Left / Right
	rotateVector(mouseDelta.x / 50.0f, camera->forward, camera->up[0], camera->up[1], camera->up[2]);
	normalize(camera->forward);

	// Up / Down
	crossProduct(vRight, camera->up, camera->forward);
	rotateVector(mouseDelta.y / 50.0f, camera->forward, 1.0f, 0.0f, 0.0f);
	rotateVector(mouseDelta.y / 50.0f, camera->up, 1.0f, 0.0f, 0.0f);
	normalize(camera->forward);
	normalize(camera->up);
*/