#include <iostream>
#include <fstream>
#include <string>
#include <cmath>

using namespace std;

class vec3
{
public:
	vec3();
	vec3(float x, float y, float z);
	void normalize();
	float magnitude();

	vec3 &operator=(const vec3 &vector);
	vec3 operator+(const vec3 &vector);
	vec3 operator-(const vec3 &vector);
	friend ostream &operator<<(ostream &, const vec3 &);

private:
	float x, y, z;
};

vec3::vec3()
{
	x = 0.0f;
	y = 0.0f;
	z = 0.0f;
}

vec3::vec3(float x, float y, float z)
{
	vec3::x = x;
	vec3::y = y;
	vec3::z = z;
}

float vec3::magnitude()
{
	return sqrt(x * x + y * y + z * z);
}

void vec3::normalize()
{
	float mag;

	mag = this->magnitude();
	if (mag)
	{
		x /= mag;
		y /= mag;
		z /= mag;
	}
}

vec3 &vec3::operator=(const vec3 &vector)
{
	x = vector.x;
	y = vector.y;
	z = vector.z;

	return *this;
}

vec3 vec3::operator+(const vec3 &vector)
{
	vec3 temp(this->x, this->y, this->z);

	temp.x += vector.x;
	temp.y += vector.y;
	temp.z += vector.z;

	return temp;
}

vec3 vec3::operator-(const vec3 &vector)
{
	vec3 temp(this->x, this->y, this->z);

	temp.x -= vector.x;
	temp.y -= vector.y;
	temp.z -= vector.z;

	return temp;
}

ostream &operator<<(ostream &output, const vec3 &vector)
{
	output << "X:" << vector.x << " Y:" << vector.y << " Z:" << vector.z << endl;
	return output;
}

int main(void)
{
	vec3 vector1(1.0f, 1.0f, 1.0f), vector2(1.0f, 2.0f, 3.0f), result;

	cout << vector1 << vector2;
	result = vector1 + vector2;
	cout << "Add " << result << endl;
	cout << vector1 << vector2;
	result = vector1 - vector2;
	cout << "Sub " << result << endl;
	cout << vector1 << vector2;
	vector1 = vector2;
	cout << "Equ " << vector1 << endl;

	vector1.normalize();
	cout << "normalize" << vector1 << endl;

	cout << "magnitude: " << 	vector1.magnitude() << endl;

	
	return 0;
}