#include "include.h"

void EngineProc(unsigned int message, int *param1, int *param2);
int EventProc(Display *, Window, GLXContext);

int main(int argc, char *argv[])
{
	Display			*display;
	XVisualInfo		*vinfo;
	XSetWindowAttributes	winattrs;
	Window			window;
	GLXContext		context;
	int attributes[] = {	GLX_RGBA, GLX_DOUBLEBUFFER,
					GLX_RED_SIZE, 8,
					GLX_GREEN_SIZE, 8,
					GLX_BLUE_SIZE, 8,
					GLX_DEPTH_SIZE, 16,
					None	};

	display	= XOpenDisplay(NULL);
	if (!display)
	{
		printf("Cannot connect to X server %s\n", XDisplayName(NULL));
		return 0;
	}
	vinfo = glXChooseVisual(display, DefaultScreen(display), attributes);
	if (!vinfo)
	{
		printf("Unable to create suitable opengl visual.\n");
		XCloseDisplay(display);
		return 0;
	}
	context	= glXCreateContext(display, vinfo, 0, True);
	if (!context)
	{
		printf("Unable to create opengl context.\n");
		XCloseDisplay(display);
		return 0;
	}

//	memset(winattrs, 0, sizeof(XSetWindowAttributes));
	winattrs.colormap	= XCreateColormap(display, DefaultRootWindow(display), vinfo->visual, AllocNone);
	winattrs.event_mask	= ExposureMask | StructureNotifyMask | KeyPressMask | PointerMotionMask | ButtonPressMask | ButtonReleaseMask;
	winattrs.border_pixel	= 0;

	window = XCreateWindow(display, DefaultRootWindow(display), 0, 0, 640, 480, 0, vinfo->depth, InputOutput, vinfo->visual,
		CWBorderPixel | CWColormap | CWEventMask, &winattrs);

	XSetStandardProperties(display, window, "xmain", "opengl", None, argv, argc, NULL );

	XMapWindow(display, window);
	glXMakeCurrent(display, window, context);

	printf("GL Version: %f\n", glGetString(GL_VERSION));

	while (True)
	{
		if ( XPending(display) )
		{
			if ( EventProc(display, window, context) )
				break;
		}
		else
		{
			EngineProc(E_RENDER, 0, 0);
		}
	}
	printf("Closing...\n");
	glXDestroyContext(display, context);
	XCloseDisplay(display);
	return 0;
}

int EventProc(Display *display, Window window, GLXContext context)
{
	XEvent		event;

	XNextEvent(display, &event);
	switch(event.type)
	{
	case CreateNotify:
		printf("CreateNotify\n");
		break;
	case ConfigureNotify:
		printf("ConfigureNotify\n");
		EngineProc(E_RESIZE, &event.xconfigure.width, &event.xconfigure.height);
		break;
	case MapNotify:
		printf("MapNotify\n");
		break;
        case ButtonPress:
		printf("ButtonPress\n");
		break;
        case ButtonRelease:
		printf("ButtonRelease\n");
		break;
        case MotionNotify:
		printf("MotionNotify\n");
		if (event.xmotion.state & (Button1Mask | Button2Mask | Button3Mask))
			EngineProc(E_MOUSEPOS, &event.xmotion.x, &event.xmotion.y);
		break;
	case KeyPress:
		printf("KeyPress\n");
		{
			KeySym keysym = XKeycodeToKeysym(display, event.xkey.keycode, 0);

		        switch (keysym)
			{
//			case XK_Delete:
				break;
			}
		}
		break;
	case KeyRelease:
		printf("KeyRelease\n");
		break;
	case UnmapNotify:
		printf("UnmapNotify\n");
		break;
	case Expose:
		{
			static init = 0;

			printf("Expose\n");

			if (!init)
				EngineProc(E_INIT, (int *)display, (int *)&window);
			init = 1;	
			if (event.xexpose.count != 0)
				break;
		}
		break;
	case DestroyNotify:
		printf("DestroyNotify\n");
		EngineProc(E_DEINIT, 0, 0);
		return 1;
	}
	return 0;
}

char *getFile(char *fileName)
{
	FILE	*file;
	char	*buffer;
	int	fSize, bRead;

	file = fopen(fileName, "rb");
	if (file == NULL)
		return 0;
	fseek(file, 0, SEEK_END);
	fSize = ftell(file);
	fseek(file, 0, SEEK_SET);
	buffer = (char *) malloc( fSize * sizeof(char) + 1 );
	bRead = fread(buffer, sizeof(char), fSize, file);
	if (bRead != fSize)
		return 0;
	fclose(file);
	buffer[fSize] = '\0';
	return buffer;
}
